#include <qapplication.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include <qmessagebox.h>
#include <stdio.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <qsplashscreen.h>
#include <qpixmap.h>

char* kcupdaterc = "# kcupdate default configuration\n\
KCSERVER=\"192.168.0.1\"\n\
PATH=\"/bin:/usr/bin:/usr/local/bin\"\n\
HOME=\"/home/local\"\n\
VERSIONFILE=\"/home/local/VERSION\"\n\
LOADPATH=\"SuSE92/kallcenter\"\n\
MAINFILE=\"kallcenter.bz2\"\n\
INSTALLFILE=\"INSTALL.update-tmp\"\n\
INDEXHTML=\"index.html\"";

char* scriptcheck = "#!/bin/sh\n\
source /etc/kcupdaterc\n\
export KCSERVER PATH HOME VERSIONFILE LOADPATH MAINFILE INSTALLFILE INDEXHTML\n\
cd /tmp/\n\
rm -f $INDEXHTML\n\
WGET=\"wget --no-proxy\"\n\
$WGET  > /dev/null\n\
if [ ! -e $INDEXHTML ]\n\
then\n\
  WGET=\"wget --proxy=off\"\n\
  $WGET http://$KCSERVER/$LOADPATH > /dev/null\n\
  if [ ! -e $INDEXHTML ]\n\
  then\n\
    echo \"No connection to update server\"\n\
    exit 1\n\
  fi\n\
fi\n\
export WGET\n\
touch /tmp/WGET\n\
chmod 600 /tmp/WGET\n\
echo -n $WGET >> /tmp/WGET\n\
grep '\"kallcenter.bz2\"' $INDEXHTML | sed 's/.* \\([0-9]*-[A-Za-z]*-20[0-9]* [0-9]*:[0-9]*\\) .*/\\1/' > /tmp/VERSION\n\
if [ ! -e $VERSIONFILE ]\n\
then\n\
  echo \"VERSION file not found\"\n\
  echo \"Not installed\" > $VERSIONFILE\n\
  exit\n\
fi\n\
if diff /tmp/VERSION $VERSIONFILE > /dev/null\n\
then\n\
  rm -f /tmp/VERSION\n\
else\n\
  exit\n\
fi\n\
rm -f $INDEXHTML\n";

char* scriptload = "#!/bin/sh\n\
source /etc/kcupdaterc\n\
export KCSERVER PATH HOME VERSIONFILE LOADPATH MAINFILE INSTALLFILE INDEXHTML\n\
cd /tmp/\n\
WGET=`cat /tmp/WGET`\n\
export WGET\n\
rm -f $INSTALLFILE\n\
if $WGET http://$KCSERVER/$LOADPATH/$INSTALLFILE\n\
then\n\
  chmod 755 $INSTALLFILE\n\
  if ./$INSTALLFILE\n\
  then\n\
    # if $INSTALLFILE returns an error, /tmp/VERSION is not deleted\n\
    mv /tmp/VERSION $VERSIONFILE\n\
  fi\n\
else\n\
  echo \"wget returned error\"\n\
  exit\n\
fi\n";

void delete_files()
{
	unlink( "/tmp/KCLOAD" );
	unlink( "/tmp/WGET" );
}

int main( int argc, char **argv )
{
	FILE* scriptfd = 0;
	printf( "kallcenter update service Version: %s %s\n", __DATE__, __TIME__);

	QApplication a( argc, argv );
	QTranslator myapp( 0 );
	myapp.load( QString( "kcupdate_" ) + QTextCodec::locale(), "/usr/local/share/apps/kallcenter/translations" );
	a.installTranslator( &myapp );

	QFont f( "sansserif", 11 );
	a.setFont( f );

	if( geteuid() ) {
		QMessageBox::critical( 0, QObject::tr( "Update not possible" ), QObject::tr( "Permissions of update service not sufficient. Please contact your administrator." ) );
		exit( 0 );
	}
	setuid( 0 );

	if( access( "/etc/kcupdaterc", F_OK ) ) { // access != 0 means: file doesn't exist
		printf( "Warning - no configuration found\n" );
		// create default configuration
		creat( "/etc/kcupdaterc", S_IRUSR|S_IWUSR );
		scriptfd = fopen( "/etc/kcupdaterc", "w" );
		fprintf ( scriptfd, "%s", kcupdaterc );
		fclose( scriptfd );
		printf( "Default configuration written to /etc/kcupdaterc\n" );
	}

	creat( "/tmp/KCUPDATE", S_IRUSR|S_IXUSR );
	scriptfd = fopen( "/tmp/KCUPDATE", "w" );
	fprintf ( scriptfd, "%s", scriptcheck );
	fclose( scriptfd );

	system( "/usr/bin/env -i sh /tmp/KCUPDATE" );
	unlink( "/tmp/KCUPDATE" );

	if( access( "/tmp/WGET", F_OK ) ) { // access != 0 means: file doesn't exist
		QMessageBox::critical( 0, QObject::tr( "Update not possible" ), QObject::tr( "No connection to kallcenter update server." ) );
		delete_files();
		exit( 0 );
	}
	if( access( "/tmp/VERSION", F_OK ) ) { // access != 0 means: file doesn't exist
		printf( "kallcenter seems up to date\n" );
		delete_files();
		exit( 0 );
	}

	QPixmap pixmap( "/usr/local/share/apps/kallcenter/icons/splash.png" );
	QSplashScreen *splash = new QSplashScreen( pixmap );
	splash->show();
	splash->message( QObject::tr( "Wait a few seconds\nwhile the software is updated." ) );
	printf( "kallcenter will be updated\n" );

	creat( "/tmp/KCLOAD", S_IRUSR|S_IXUSR );
	scriptfd = fopen( "/tmp/KCLOAD", "w" );
	fprintf ( scriptfd, "%s", scriptload );
	fclose( scriptfd );

	system( "/usr/bin/env -i sh /tmp/KCLOAD" );
	if( !access( "/tmp/VERSION", F_OK ) ) { // access == 0 means: file was not deleted
		QMessageBox::critical( 0, QObject::tr( "Update failed" ), QObject::tr( "Load script returned with error - please notify your administrator." ) );
		unlink( "/tmp/VERSION" );
	}

	delete_files(); 

	splash->close();
	delete splash;
}
